@@ -1,5 +1,5 @@
 package TreePath;
-$TreePath::VERSION = '0.16';
+$TreePath::VERSION = '0.17';
 use utf8;
 use Moose;
 with 'MooseX::Object::Pluggable';
@@ -22,31 +22,31 @@ has conf => ( is => 'rw',
               isa => 'MyConf',
               coerce => 1,
               trigger  => sub {
-                my $self = shift;
-                my $args = shift;
+                  my $self = shift;
+                  my $args = shift;
 
-                # if conf exist
-                if ( defined $args->{$self->configword} ) {
-                  croak "Error: Can not find " . $self->configword . " in your conf !"
-                    if ( ! $args->{$self->configword});
+                  # if args exist
+                  if ( defined $args->{$self->configword} ) {
+                      croak "Error: Can not find " . $self->configword . " in your conf !"
+                          if ( ! $args->{$self->configword});
 
-                  $self->config($args->{$self->configword});
+                      $self->config($args->{$self->configword});
 
-                  $self->debug($self->config->{'debug'})
-                    if ( ! defined $self->debug && defined $self->config->{'debug'} );
+                      $self->debug($self->config->{'debug'})
+                          if ( ! defined $self->debug && defined $self->config->{'debug'} );
 
-                  $self->_load_backend if ! $self->can('backend');
-                }
-                # it's a hash
-                else {
-                  $self->tree($args);
-                  $self->_build_tree;
-                }
+                      my $backend = $self->config->{'backend'}->{name};
+                      $self->backend($backend);
+                  }
               }
             );
 
 
 
+has datas => ( is => 'rw',
+              isa => 'ArrayRef',
+          );
+
 has config => (
                isa      => "HashRef",
                is       => "rw",
@@ -61,9 +61,16 @@ has 'debug' => (
                 is       => 'rw',
                );
 
-has '_backend' => (
+has 'backend' => (
                 is       => 'rw',
                 isa      => 'Str',
+                trigger  => sub {
+                    my $self    = shift;
+                    my $backend = shift;
+
+                    $self->_log("Loading $backend backend ...");
+                    $self->load_plugin( $backend );
+                    }
                );
 
 has _plugin_ns => (
@@ -73,28 +80,11 @@ has _plugin_ns => (
                 default  => sub{ 'Backend' },
                   );
 
-has _source_field => (
-                is       => 'rw',
-                isa      => 'Str',
-                default  => sub{ 'source' },
-                  );
-
-has _default_source => (
-                is       => 'rw',
-                isa      => 'Str',
-                default  => sub{ 'Page' },
-                  );
-
-has _search_field => (
-                is       => 'rw',
-                isa      => 'Str',
-                default  => sub{ 'name' },
-                  );
-
 has tree => (
                 isa      => "HashRef",
                 is       => "rw",
-);
+                default  => sub {{}},
+            );
 
 
 has root => (
@@ -104,26 +94,6 @@ has root => (
 
 
 
-sub _load_backend {
-  my $self = shift;
-  my $backend = $self->config->{'backend'}->{name};
-  $self->_backend($backend);
-
-  $self->_log("Loading $backend backend ...");
-  $self->load_plugin( $backend );
-  $self->_load_tree;
-  $self->_build_tree;
-}
-
-sub reload {
-    my $self = shift;
-
-    $self->_populate_backend(0)
-        if $self->can('_populate_backend');
-
-    $self->_load_tree;
-}
-
 sub _log{
   my ($self, $msg ) = @_;
 
@@ -132,117 +102,46 @@ sub _log{
   say STDERR "[debug] $msg";
 }
 
-# Load tree from backend
-sub _load_tree {
-  my $self = shift;
+# this method is overridden if a backend is loaded
+sub _load{
+    my $self  = shift;
 
-  $self->tree($self->_load);
+    return $self->datas;
 }
 
-# Build Tree
-# build root and relationships (just belongs_to)
-sub _build_tree {
-  my $self = shift;
 
-  my $tree = $self->tree;
+sub load{
+    my $self  = shift;
 
-  foreach my $id ( sort { $a cmp $b } keys %$tree ) {
+    my $nodes = $self->_load;
 
-
-      my $node = $self->tree->{$id};
-
-      # transform obj_id to object reference
-      foreach my $col ( keys %$node ) {
-
-          # hash = belongs_to
-          if ( ref($node->{$col}) eq 'HASH') {
-
-              my @values = keys %{$node->{$col}};
-              my $value = $values[0];
-
-              if ( $value =~ m/^\w*_\d*$/ ) {
-                  $node->{$col} = $self->tree->{$value};
-              }
-              else {
-                  die "Error: Relationship belongs_to is not well formatted : $value, use Obj_ID as form"
-              }
-          }
-          # array = has_may
-          elsif ( ref($node->{$col}) eq 'ARRAY') {
-              my $newarray = [];
-              foreach my $child (@{$node->{$col}}) {
-                  if ( $child =~ m/^\w*_\d*$/ ) {
-                      push(@$newarray, $self->tree->{$child});
-                  }
-                  elsif ( ref($child) eq 'HASH') {
-                      push(@$newarray, $child);
-                  }
-                  else {
-                      die "Error : has_many relationship $col (=$child) is not formatted, ex: 'Page_3'\n";
-                  }
-              }
-              $node->{$col} = $newarray;
-          }
-      }
-
-      my $parent_field = $self->_parent_field($node);
-      $self->add($node, $node->{$parent_field});
-  }
+    $nodes = $self->_load_order($nodes);
+    foreach my $node (@$nodes) {
+        $self->add($node);
+    }
 }
 
-sub _parent_field{
-    my $self = shift;
-    my $node = shift;
+sub _load_order{
+    my $self  = shift;
+    my $nodes = shift;
 
-    my $source;
-    if ( ref($node)) {
-        $source = $node->{$self->_source_field};
+    my @nodes_ordered;
+    if ( defined $self->config && defined $self->config->{backend} && defined $self->config->{backend}->{args}) {
+        my $load_order = $self->config->{backend}->{args}->{load_order};
+        if ( defined $load_order) {
+            foreach my $source ( @$load_order ) {
+                foreach my $node ( @$nodes ) {
+                    if ( $node->{$self->_get_key_name('source', $node)} eq $source) {
+                        push(@nodes_ordered, $node);
+                    }
+                }
+            }
+            return \@nodes_ordered;
+        }
     }
-    else {
-        $source = $node;
-    }
-    my $parent_field;
-    if ( defined $self->config && defined $self->config->{backend}->{args}->{sources_name}->{$source}->{parent_field}) {
-        $parent_field = $self->config->{backend}->{args}->{sources_name}->{$source}->{parent_field};
-    }
-    else {
-        $parent_field = 'parent';
-    }
-    return $parent_field;
+
+    return $nodes;
 }
-# sub  _bckd_create {
-#     my $self = shift;
-#     my $node = shift;
-#     my $msg  = shift;
-
-#     return if ( ! $self->_backend || ! $self->_sync );
-#     $self->_log("[" . $self->_backend . "] CREATE " . $node->{name} . " | $msg");
-#     $self->_create($node);
-# }
-
-# sub  _bckd_delete {
-#     my $self  = shift;
-#     my $nodes = shift;
-#     my $msg   = shift;
-
-#     return if ( ! $self->_backend || ! $self->_sync );
-#     my @nodes_name = map { $_->{$self->_search_field} } @$nodes;
-#     $self->_log("[" . $self->_backend . "] DELETE @nodes_name | $msg");
-
-#     $self->_delete($nodes);
-# }
-
-# sub  _bckd_update {
-#     my $self = shift;
-#     my $node = shift;
-#     my $msg  = shift;
-
-#     return if ( ! $self->_backend || ! $self->_sync );
-#     $self->_log("[" . $self->_backend . "] UPDATE " . $node->{name} . " | $msg");
-#     $self->_update($node);
-# }
-
-
 
 
 # removes the child's father
@@ -250,12 +149,12 @@ sub _remove_child_father{
     my $self = shift;
     my $node = shift;
 
-    my $father = $node->{parent};
+    my $father = $node->{$self->_get_key_name('parent', $node)};
     my $key_children = $self->_key_children($node, $father);
     my $id = 0;
     foreach my $child ( @{$father->{$key_children}}) {
-        if ( $child->{$self->_search_field} eq $node->{$self->_search_field} &&
-                 $child->{parent} eq $node->{parent} ){
+        if ( $child->{$self->_get_key_name('search', $child)} eq $node->{$self->_get_key_name('search', $node)} &&
+                 $child->{$self->_get_key_name('parent', $child)} eq $node->{$self->_get_key_name('parent', $node)} ){
             return splice ( @{$father->{$key_children}},$id,1);
         }
         $id++;
@@ -263,32 +162,11 @@ sub _remove_child_father{
 }
 
 
-# sub _clone_node {
-#     my $self = shift;
-#     my $node = shift;
-
-#     my $clone = {};
-#     foreach my $k (keys %$node) {
-#         if ( $k eq 'parent'){
-#             if ( $node->{'parent'}) {
-#                 $clone->{$self->_parent_field} = $node->{$k}->{id};
-#             }
-#             # is root
-#             else {
-#                 $clone->{$self->_parent_field} = 0;
-#             }
-#         }
-#         else { $clone->{$k} = $node->{$k} }
-#     }
-#     return $clone;
-# }
-
-
-sub _key_obj {
+sub _obj_key {
   my $self = shift;
   my $node = shift;
 
-  return $node->{source} . '_' . $node->{id};
+  return $node->{$self->_get_key_name('source', $node)} . '_' . $node->{$self->_get_key_name('primary', $node)};
 }
 
 
@@ -297,38 +175,20 @@ sub _key_children {
     my ($self, $node, $parent) = @_;
 
     my $key_children;
-    #if parent have a source
-    if ( defined $parent->{$self->_source_field}) {
+    my $node_source_key   = $self->_get_key_name('source', $node);
+    my $parent_source_key = $self->_get_key_name('source', $parent);
 
-        # if node have a source
-        if ( defined $node->{$self->_source_field}) {
-
-            # if node and parent have the same source
-            if ( $node->{$self->_source_field} eq $parent->{$self->_source_field} ) {
-                $key_children = 'children';
-            }
-            else {
-                $key_children = 'children_' . $node->{$self->_source_field};
-            }
-        }
-        # else node have the same source as parent
-        else {
-            $node->{$self->_source_field} = $parent->{$self->_source_field};
-            $key_children = 'children';
-        }
+    # if node and parent have the same source
+    if ( $node->{$node_source_key} eq $parent->{$parent_source_key} ) {
+        $key_children = 'children';
     }
-    # parent haven't source
     else {
-        if ( defined $node->{$self->_source_field}) {
-            die "node " . $node->{$self->_search_field} . " [id:". $node->{id} . "] have a source but not the parent [id:" . $parent->{id} . "]!";
-        }
-        else {
-            $key_children = 'children';
-        }
+        $key_children = 'children_' . $node->{$node_source_key};
     }
     return $key_children;
 }
 
+
 sub search {
   my ( $self, $args ) = @_;
 
@@ -338,44 +198,42 @@ sub search {
 
       my $found = 1;
 
-      # next search source field and it's not same source'
-      next if ( $args->{$self->_source_field}
-                    && $tree->{$id}->{$self->_source_field} ne $args->{$self->_source_field}
-                );
 
-    foreach my $key ( keys %$args ) {
-      my $current;
-      if ( $key =~ m/(.*)\.(.*)/) {
-        # ex: parent.name
-        if ( defined $tree->{$id}->{$1} && ref($tree->{$id}->{$1})) {
-          $current = $tree->{$id}->{$1}->{$2};
-        }
-        else { next }
-      }
-      else {
-          if ( defined $tree->{$id}->{$key} ){
-              $current = $tree->{$id}->{$key};
+      foreach my $key ( keys %$args ) {
+          my $current;
+          if ( $key =~ m/(.*)\.(.*)/) {
+              # ex: parent.name
+              if ( defined $tree->{$id}->{$1} && ref($tree->{$id}->{$1})) {
+                  $current = $tree->{$id}->{$1}->{$2};
+              }
+              else { next }
           }
           else {
-              die "'$key' is not a key hash [node obj_key:$id]";
+              if ( defined $tree->{$id}->{$key} ){
+                  $current = $tree->{$id}->{$key};
+              }
+              else {
+                  #print "the '$key' key is unknown [node obj_key:$id]\n";
+                  $found = 0;
+                  next;
+              }
+          }
+          my $value = $args->{$key};
+          if ( $current ne $value ) {
+              $found = 0;
+              last;
           }
       }
-      my $value = $args->{$key};
-      if ( $current ne $value ) {
-        $found = 0;
-        last;
-      }
-    }
 
-    if ( $found ){
-      if ( wantarray) {
-        push(@$results, $tree->{$id});
+      if ( $found ){
+          if ( wantarray) {
+              push(@$results, $tree->{$id});
+          }
+          # if found and scalar context
+          else {
+              return $tree->{$id};
+          }
       }
-      # if found and scalar context
-      else {
-        return $tree->{$id};
-      }
-    }
   }
 
   return 0 if (  ! wantarray && ! $$results[0] );
@@ -388,36 +246,27 @@ sub search {
 # ex : search_path(/A/B/C')
 #      or search_path(/A/B/C, { by => 'name', source => 'Page'} )
 sub search_path {
-  my ( $self, $path, $opts ) = @_;
-
-
-  # search by 'name' if not defined
-  $opts->{by} = $self->_search_field if ! defined $opts->{by};
-
-  # source => 'Page' if not defined
-  $opts->{$self->_source_field} = $self->_default_source if ! defined $opts->{$self->_source_field};
-
+  my ( $self, $source, $path ) = @_;
 
   croak "path must be start by '/' !: $!\n" if ( $path !~ m|^/| );
 
+  my $search_key = $self->_get_key_name('search', { source => $source});
+
   my $nodes = [ split m%/%, $path ];
   $$nodes[0] = '/';
 
+  my $not_found = 0;
   my (@found, @not_found);
-  my $parent = '/';
   foreach my $node ( @$nodes ) {
 
-    my $args = { $opts->{by} => $node, "parent\.$opts->{by}" => $parent, source => $opts->{source}};
-    my $result = $self->search( $args );
+    my $result = $self->search({ $search_key => $node, source => $source } );
 
-    $parent = $result->{$opts->{by}} if $result;
-
-    if ( $result ) {
-      push(@found, $result);
+    if ( ! $not_found && $result ) {
+        push(@found, $result);
     }
     else {
-
-      push(@not_found, $node);
+        $not_found = 1;
+        push(@not_found, $node);
     }
   }
 
@@ -506,28 +355,119 @@ sub del {
       # traverse child branches and delete it
       my $nodes = $self->traverse($node);
 
-      push(@deleted,map { delete $self->tree->{$self->_key_obj($_)} } @$nodes);
+      push(@deleted,map { delete $self->tree->{$self->_obj_key($_)} } @$nodes);
       #$self->_bckd_delete($nodes, "delete " . @$nodes . " node(s)");
   }
   return @deleted;
 }
 
+
+
+sub _get_key_name {
+  my ($self, $key, $node) = @_;
+
+  my $values = {
+      source => 'source',
+      primary => 'id',
+      parent => 'parent',
+      search => 'name',
+  };
+
+  die "the key '$key' is unknown in _get_key_name !"
+      if ! defined $values->{$key};
+
+  if ( defined $self->config ) {
+      return $self->config->{backend}->{args}->{sources_name}->{$node->{source}}->{"${key}_key"} ||
+          $values->{$key};
+  }
+  return $values->{$key};
+  };
+
+
+# checks that the necessary keys are set
+sub _check_node {
+  my ($self, $node, $parent) = @_;
+
+  # check parent key
+  my $primary_key =  $self->_get_key_name('primary', $node);
+  die "key $primary_key is not set in node !"
+      if ! defined $node->{$primary_key};
+
+  # check parent key
+  if ( ! $parent ) {
+      my $parent_key =  $self->_get_key_name('parent', $node);
+      die "key $parent_key is not set in node !"
+          if ! defined $node->{$parent_key} && $self->root;
+  }
+}
+
+
 # Inserts a node beneath the parent
 sub add {
   my ($self, $node, $parent) = @_;
 
+  $self->_log("Add node ". $self->_obj_key($node));
 
-  if ( ! $parent ){
-      if ( $self->root) {
-          die "root already exist [ node: " .$node->{$self->_source_field}. '_' . $node->{id} . " ]!";
-      }
-      $self->root($node);
+  $self->_check_node($node, $parent);
+  my $parent_key = $self->_get_key_name('parent', $node);
+
+  if ( $parent ) {
+      $node->{$parent_key} = $parent;
   }
 
-  $node->{parent} = $parent;
+  # transform obj_id to object reference
+  foreach my $col ( keys %$node ) {
+
+      # hash = belongs_to
+      if ( ref($node->{$col}) eq 'HASH') {
+
+          # if $parent and col == parent
+          if ( defined $parent && $col eq $parent_key) {
+              $node->{$col} = $parent;
+          }
+          else {
+              my @values = keys %{$node->{$col}};
+              my $value = $values[0];
+
+              if ( $value =~ m/^\w*_\d*$/ ) {
+                  $node->{$col} = $self->tree->{$value};
+              }
+              else {
+                  die "Error: Relationship belongs_to is not well formatted : $value, use Obj_ID as form"
+              }
+          }
+      }
+      # array = has_may
+      elsif ( ref($node->{$col}) eq 'ARRAY') {
+          my $newarray = [];
+          foreach my $child (@{$node->{$col}}) {
+              if ( $child =~ m/^\w*_\d*$/ ) {
+                  push(@$newarray, $self->tree->{$child});
+              }
+              elsif ( ref($child) eq 'HASH') {
+                  push(@$newarray, $child);
+              }
+              else {
+                  die "Error : has_many relationship $col (=$child) is not formatted, ex: 'Page_3'\n";
+              }
+          }
+          $node->{$col} = $newarray;
+      }
+
+      # col eq 'parent' or equivalent
+      if ( $col eq $parent_key ) {
+          $parent = $node->{$parent_key};
+          if ( ! $parent ){
+              if ( $self->root) {
+                  die "root already exist [ node: " .$node->{$self->_get_key_name('source', $node)}. '_' . $node->{$self->_get_key_name('primary', $node)} . " ]!";
+              }
+              $self->root($node);
+          }
+      }
+  }
 
   # save ref node in tree
-  $self->tree->{$self->_key_obj($node)} = $node;
+  $self->tree->{$self->_obj_key($node)} = $node;
 
   my $key_children;
   my $update_children = 0;
@@ -535,16 +475,11 @@ sub add {
   # node != root
   if ( defined $parent && $parent ) {
       $key_children = $self->_key_children($node, $parent);
-
       if ( $parent ) {
           push(@{$parent->{$key_children}}, $node);
       }
   }
-  # add node in backend
-  #$self->_bckd_create($node, 'add node ' . $node->{$self->_search_field})
-  #    if ! $is_exist_in_backend;
 
-  return $self->root if ! $parent;
 
   return $node;
 }
@@ -574,7 +509,7 @@ sub update {
 
           $node->{parent} = $new_parent;
 
-          my $msg = 'updating parent of '. $node->{$self->_search_field} .
+          my $msg = 'updating parent of '. $node->{$self->_get_key_name('search',$node)} .
               " ( " . $old->{name} . " -> ". $node->{parent}->{name} .")";
 
           #$self->_bckd_update($node, $msg);
@@ -597,6 +532,8 @@ sub move {
 
 TreePath - Simple Tree Path!
 
+This module is at EXPERIMENTAL stage, so use with caution.
+
 =head1 VERSION
 
 
@@ -605,6 +542,10 @@ TreePath - Simple Tree Path!
  use TreePath;
 
  my $tp = TreePath->new(  conf  => $conf  );
+
+ # load datas from backend
+ $tp->load;
+
  my $tree = $tp->tree;
 
  # All nodes are hash
@@ -625,42 +566,49 @@ TreePath - Simple Tree Path!
  $tp = TreePath->new( conf => 't/conf/treefromdbix.yml')
 
  # see t/conf/treepath.yml for hash structure
- $tp = TreePath->new( conf => $datas);
+ $tp = TreePath->new( datas => $datas);
 
  also see t/01-tpath.t
 
 =cut
 
+=head2 load
+
+ # The data load is not automatic
+ $tp->load;
+
+=cut
+
 =head2 tree
 
  $tree = $tp->tree;
 
 =cut
 
-=head2 reload
-
- # reload tree from backend
- $tree = $tp->reload;
-
-=cut
-
 =head2 nodes
 
  $root = $tp->root;
- # $root and $tree->{1} are the same node
 
  This is the root node ( a simple hashref )
  it has no parent.
-     {
-       'Page_1' => {
+
+               {
                 'id' => '1',
                 'name' => '/',
                 'parent' => '0'
                 'source' => 'Page'
               }
-     }
 
-  $A = $tp->search( { name => 'A', source => 'Page' } );
+  All keys can be a relation to another source, as belong_to with DBIx like this :
+               {
+                'id' => '15',
+                'name' => '/',
+                'parent' => { 'Page_10' }
+                'source' => 'Comment'
+              }
+
+  where 'Page' is the source of relationship and '10' the id. So Comment_15 is linked with Page_10.
+
   See the dump :
 
     {
@@ -694,12 +642,7 @@ TreePath - Simple Tree Path!
      'source' => 'Page'
     }
 
-    => 'parent' is a reference on root node and 'children' is an array containing 2 nodes
-
-    => if nodes have 'source', it's possible to 'link' some 'source' of node on some others sources
-
-       For example, A:(source1), B:(source1, parent:A), C:(source2, parent:A) becomes :
-       A:(source1, children:[B], children_source2[C])
+    => 'parent' is a reference to root node and 'children' is an array containing 2 nodes
 
 =cut
 
@@ -716,14 +659,14 @@ TreePath - Simple Tree Path!
 
 =cut
 
-=head2 search_path (PATH)
+=head2 search_path (SOURCE, PATH)
 
  # Search a path in a tree
- # in scalar context return last node
- my $c = $tp->search_path('/A/B/C');
+ # in scalar context return last node (last page in this example)
+ my $c = $tp->search_path('Page', '/A/B/C');
 
  # in array context return found and not_found nodes
- my ($found, $not_found) = $tp->search_path('/A/B/X/D/E');
+ my ($found, $not_found) = $tp->search_path('Page', '/A/B/X/D/E');
 
 =cut
 
@@ -764,10 +707,10 @@ TreePath - Simple Tree Path!
 
 =cut
 
-=head2 add ($node, $parent)
+=head2 add ($node, [ $parent ])
 
  # add the root
- $root = $tp->add({ name => '/', source => 'Node' }, 0);
+ $root = $tp->add({ name => '/', source => 'Node', id => 123 });
 
  # add a node beneath the parent.
  $Z = $tp->add({ name => 'Z', source => 'Node' }, $parent);
@@ -790,14 +733,6 @@ TreePath - Simple Tree Path!
 
 =cut
 
-=head2 insert_before ($sibling, $node)
-
- # Inserts a node beneath the parent before the given sibling.
- $Y = $tp->insert_before($Z, { name => 'Y' });
-
-=cut
-
-
 
 =head1 AUTHOR
 
