@@ -1,5 +1,5 @@
 package TreePath::Backend::DBIx;
-$TreePath::Backend::DBIx::VERSION = '0.16';
+$TreePath::Backend::DBIx::VERSION = '0.17';
 use Moose::Role;
 use base 'DBIx::Class::Schema';
 use Carp qw/croak/;
@@ -50,19 +50,16 @@ has '_populate_backend' => (
                 isa      => 'Int',
                );
 
-
 sub _build_model_config {
   my $self = shift;
-  my $config       = $self->conf;
 
-  my $model_config = $config->{$self->config->{backend}->{args}->{model}}
-    or croak "'backend/args/model' is not defined in conf file !";
+  my $model_config = $self->conf->{$self->config->{backend}->{args}->{model}}
+      or croak "'backend/args/model' is not defined in conf file !";
   return $model_config
 }
 
 sub _build_schema {
   my $self = shift;
-  my $config       = $self->conf;
 
   my($dsn, $user, $password, $allattrs) = $self->_connect_info;
 
@@ -74,6 +71,7 @@ sub _build_schema {
   return $schema_class->connect($dsn,$user,$password,$allattrs);
 }
 
+
 sub _connect_info {
   my $self = shift;
 
@@ -138,7 +136,7 @@ sub _load {
     my @sources_name = keys %{$self->config->{backend}->{args}->{sources_name}}
         or croak "'backend/args/sources_name' is not defined in conf file !";
 
-    my $datas = {};
+    my @datas;
    foreach my $source_name ( @sources_name ) {
 
        my($dsn, $user, $password, $allattrs) = $self->_connect_info;
@@ -150,7 +148,6 @@ sub _load {
           $schema->deploy;
           $schema->_populate if ( $schema->can('_populate') && $self->_populate_backend);
       }
-      $self->schema($schema);
       my $rs = $self->schema->resultset($source_name)->search();
 
        my @primary_columns = $rs->result_source->primary_columns;
@@ -161,10 +158,10 @@ sub _load {
        foreach my $row ( $rs->all) {
            my $obj = $self->_row_to_obj($row);
            $obj->{$primary_key} = $row->$primary_key;
-           $datas->{$self->_obj_key($row)} = $obj;
+           push(@datas,$obj);
        }
    }
-    return $datas;
+    return \@datas;
 }
 
 
@@ -189,19 +186,19 @@ sub _row_to_obj {
         if ( ! ref($value)) {
             $obj->{$col} = $value;
         }
-        # relationship has_many
+        # relationship has_many (not already tested)
         elsif ( ref($value) eq 'DBIx::Class::ResultSet') {
             my @results = $row->$col;
             $obj->{$col} = [];
             #  save rs in array
             foreach my $res (@results) {
-                push( $obj->{$col}, $self->_obj_key($res));
+                push( @{$obj->{$col}}, $self->_rs_to_obj_key($res));
             }
         }
         # relationship belongs_to
         else {
             # save result in hashref
-            $obj->{$col} = { $self->_obj_key($row->$col) => 1 };
+            $obj->{$col} = { $self->_rs_to_obj_key($row->$col) => 1 };
         }
     }
     # Ajout de la source a l'objet
@@ -210,7 +207,7 @@ sub _row_to_obj {
 }
 
 
-sub _obj_key {
+sub _rs_to_obj_key {
     my $self = shift;
     my $row  = shift;
 
@@ -253,7 +250,7 @@ TreePath::Backend::DBIx - Backend 'DBIx' for TreePath
 
 =head1 VERSION
 
-version 0.16
+version 0.17
 
 =head1 CONFIGURATION
 
