@@ -1,8 +1,9 @@
 package TreePath::Backend::File;
-$TreePath::Backend::File::VERSION = '0.16';
+$TreePath::Backend::File::VERSION = '0.17';
 
 use Moose::Role;
 use Config::JFDI;
+use YAML qw(LoadFile);
 use Carp qw/croak/;
 
 sub _load {
@@ -14,9 +15,12 @@ sub _load {
 
   $self->_log("Loading tree from file $file");
 
-  my ($jfdi_h, $jfdi) = Config::JFDI->open($file)
-    or croak "Error (conf: $file) : $!\n";
-  return $jfdi->get;
+  # Can not use a arrayref with this, only hashref :(
+  # my ($jfdi_h, $jfdi) = Config::JFDI->open($file)
+  #    or croak "Error (conf: $file) : $!\n";
+  #  return $jfdi->get;
+
+  return LoadFile($file);
 }
 
 sub _create {
@@ -43,7 +47,7 @@ TreePath::Backend::File - Backend 'File' for TreePath
 
 =head1 VERSION
 
-version 0.16
+version 0.17
 
 =head1 CONFIGURATION
 
